import {TgHttp} from "@coreHttp";
import apis from "@appTiangong/apis";
import localStorage from "@core/utils/TgLocalStorage";
import {useStore} from "vuex";
import {reactive, toRefs, inject} from "vue";
import {SettingCore} from "@core/setting";

export default function () {
    const store = useStore();
    const state = reactive({
        NoReadAllList: [],//全部未读数量
        noReadTop10List: [],//前10的未读数量
        allNoticeList: [],//全部的消息列表

    })
    let messageBox = inject('tg-message-box')

    /**
     * 获取用户的未读消息数量，存储localStorage和vuex中
     * @returns {Promise<void>}
     */
    const getNoticeNumber = async () => {
        if (website.tgNoticeEnable) {
            let request = new TgHttp();
            let response = await request.build(apis.tgMessageRead.messageCount).post();
            if (response.success) {
                let data = response.data.unReadCount;
                // 保存到vuex的变量中
                store.dispatch("updateNoticeNumber", data);
                //存到localStorage中,因为vuex变量，在界面刷新的时候会丢。
                localStorage.set("updateNoticeNumber", data);
            }
        }
    }

    /**
     * 获取未读消息列表
     */
    async function getNoticeList() {
        let entity = {
            readStatus: '0'
        }
        let paging = {
            pageSize: 0,
            pageNum: 1,
            sortFields: [{sortField: "createTime", asc: false}]
        }
        let response = await getMessageList(entity, paging);
        if (response.success) {
            if (response.data && response.data.list) {
                let noReadNoticeList = []
                response.data.list.forEach(item => {
                    noReadNoticeList.push(item)
                })
                state.NoReadAllList = noReadNoticeList
                //界面只显示10条未读
                state.noReadTop10List = noReadNoticeList.slice(0, 10)
            }
        }

    }

    /**
     * 全部已读
     * @returns {Promise<void>}
     */
    const setAllRead = async () => {
        if (state.NoReadAllList == undefined || state.NoReadAllList.length <= 0) {
            return tg.msg.error("当前没有未读消息")
        }
        let request = new TgHttp();
        let response = await request.build(apis.tgMessageRead.readAll).post()
        if (response.success) {
            //设置vuex未读数量为0
            store.dispatch("updateNoticeNumber", 0);
            //清空localStorage
            localStorage.remove("updateNoticeNumber");
            //刷新消息列表
            getNoticeList()
        }
    }

    /**
     * 单条消息已读
     * @returns {Promise<void>}
     */
    const setReadById = async (id) => {
        if (id == undefined || id == null) {
            return tg.msg.error("当前没有选中需要读取的消息")
        }
        let request = new TgHttp();
        request.entity.id = id
        // let response =
        await request.build(apis.tgMessageRead.read).post()
        // if (response.success) {
        //     tg.msg.success('')
        // }
    }

    /**
     * 获取消息列表
     * @param entity
     * @param paging
     * @returns {Promise<TgResponse>}
     */
    const getMessageList = async (entity, paging) => {
        let req = new TgHttp();
        req.entity = entity;
        req.paging = paging;
        let res = await req.build(apis.tgMessageRead.messageList).post();
        return res;
    }

    /**
     * 获取小图标的描述信息
     * @param item
     * @returns {string}
     */
    const getMsgStyle = () => {
        // if (item == 'primary') {
        //     return '通知'
        // } else if (item == 'success') {
        //     return '通知'
        // } else if (item == 'warning') {
        //     return '警告'
        // }
        return '通知'
    }

    /**
     * 创建sse连接
     */
    const createSseConnect = () => {
        let enable = false
        if (website.tgNoticeEnable) {
            enable = website.tgNoticeEnable;
        }
        //如果没有启用，就不会创建通道
        if (!enable) {
            return;
        }
        if (window.EventSource) {
            let token = localStorage.get("token");
            if (token) {
                let baseUrl = window.website.apiServer.base || SettingCore.VITE_BASE_URL;

                let api = baseUrl + "/sse/connect?token=" + token;
                const eventSource = new EventSource(api);

                eventSource.onmessage = (event) => {
                };

                eventSource.onopen = (event) => {
                };

                eventSource.onerror = (event) => {
                };

                eventSource.close = (event) => {
                };

                eventSource.addEventListener("heart", (e) => {
                })

                eventSource.addEventListener("notice", (e) => {
                    let noticeData = JSON.parse(e.data)
                    //1,修改提示的数字
                    store.dispatch("updateNoticeNumber", noticeData.unReadCount);
                    localStorage.set("updateNoticeNumber", noticeData.unReadCount);

                    // 2,如果要强制阅读，弹框进行阅读
                    if (noticeData.forcedRead == '1') {
                        let second = noticeData.forcedReadSec
                        let title = noticeData.messageTitle
                        let content = noticeData.messageContent

                        let instance = messageBox({
                            title: title,
                            content: content,
                            delaySec: second
                        })
                        //调用chooseList的show
                        if (instance && instance.exposed) {
                            instance.exposed.show()
                        }
                    }
                    // console.log('notice:' + noticeData)
                })
            }
        } else {
            console.log("你的浏览器不支持SSE~")
        }
    }

    return {
        ...toRefs(state),
        getNoticeNumber,
        getNoticeList,
        createSseConnect,
        setAllRead,
        setReadById,
        getMsgStyle
    };
}




